@@ -89,7 +89,7 @@ describe "Regexp with character classes" do
     /[^[:lower:]A-C]+/.match("abcABCDEF123def").to_a.should == ["DEF123"] # negated character class
     /[:alnum:]+/.match("a:l:n:u:m").to_a.should == ["a:l:n:u:m"] # should behave like regular character class composed of the individual letters
     /[\[:alnum:]+/.match("[:a:l:n:u:m").to_a.should == ["[:a:l:n:u:m"] # should behave like regular character class composed of the individual letters
-    lambda { eval('/[[:alpha:]-[:digit:]]/') }.should raise_error(SyntaxError) # can't use character class as a start value of range
+    -> { eval('/[[:alpha:]-[:digit:]]/') }.should raise_error(SyntaxError) # can't use character class as a start value of range
   end
 
   it "matches ASCII characters with [[:ascii:]]" do
@@ -562,6 +562,13 @@ describe "Regexp with character classes" do
     "\u{16EE}".match(/[[:word:]]/).to_a.should == ["\u{16EE}"]
   end
 
+  ruby_bug "#19417", ""..."3.4.6" do
+    it "matches Unicode join control characters with [[:word:]]" do
+      "\u{200C}".match(/[[:word:]]/).to_a.should == ["\u{200C}"]
+      "\u{200D}".match(/[[:word:]]/).to_a.should == ["\u{200D}"]
+    end
+  end
+
   it "doesn't match Unicode No characters with [[:word:]]" do
     "\u{17F0}".match(/[[:word:]]/).should be_nil
   end
@@ -609,25 +616,32 @@ describe "Regexp with character classes" do
     "루비(Ruby)".match(/\p{Hangul}+/u).to_a.should == ["루비"]
   end
 
-  ruby_version_is "2.4" do
-    it "supports \\X (unicode 9.0 with UTR #51 workarounds)" do
-      # simple emoji without any fancy modifier or ZWJ
-      /\X/.match("\u{1F98A}").to_a.should == ["🦊"]
+  it "supports negated property condition" do
+    "a".match(eval("/\P{L}/")).should be_nil
+    "1".match(eval("/\P{N}/")).should be_nil
+  end
 
-      # skin tone modifier
-      /\X/.match("\u{1F918}\u{1F3FD}").to_a.should == ["🤘🏽"]
+  it "raises a RegexpError for an unterminated unicode property" do
+    -> { Regexp.new('\p{') }.should raise_error(RegexpError)
+  end
 
-      # emoji joined with ZWJ
-      /\X/.match("\u{1F3F3}\u{FE0F}\u{200D}\u{1F308}").to_a.should == ["🏳️‍🌈"]
-      /\X/.match("\u{1F469}\u{200D}\u{1F469}\u{200D}\u{1F467}\u{200D}\u{1F466}").to_a.should == ["👩‍👩‍👧‍👦"]
+  it "supports \\X (unicode 9.0 with UTR #51 workarounds)" do
+    # simple emoji without any fancy modifier or ZWJ
+    /\X/.match("\u{1F98A}").to_a.should == ["🦊"]
 
-      # without the ZWJ
-      /\X+/.match("\u{1F3F3}\u{FE0F}\u{1F308}").to_a.should == ["🏳️🌈"]
-      /\X+/.match("\u{1F469}\u{1F469}\u{1F467}\u{1F466}").to_a.should == ["👩👩👧👦"]
+    # skin tone modifier
+    /\X/.match("\u{1F918}\u{1F3FD}").to_a.should == ["🤘🏽"]
 
-      # both of the ZWJ combined
-      /\X+/.match("\u{1F3F3}\u{FE0F}\u{200D}\u{1F308}\u{1F469}\u{200D}\u{1F469}\u{200D}\u{1F467}\u{200D}\u{1F466}")
-        .to_a.should == ["🏳️‍🌈👩‍👩‍👧‍👦"]
-    end
+    # emoji joined with ZWJ
+    /\X/.match("\u{1F3F3}\u{FE0F}\u{200D}\u{1F308}").to_a.should == ["🏳️‍🌈"]
+    /\X/.match("\u{1F469}\u{200D}\u{1F469}\u{200D}\u{1F467}\u{200D}\u{1F466}").to_a.should == ["👩‍👩‍👧‍👦"]
+
+    # without the ZWJ
+    /\X+/.match("\u{1F3F3}\u{FE0F}\u{1F308}").to_a.should == ["🏳️🌈"]
+    /\X+/.match("\u{1F469}\u{1F469}\u{1F467}\u{1F466}").to_a.should == ["👩👩👧👦"]
+
+    # both of the ZWJ combined
+    /\X+/.match("\u{1F3F3}\u{FE0F}\u{200D}\u{1F308}\u{1F469}\u{200D}\u{1F469}\u{200D}\u{1F467}\u{200D}\u{1F466}")
+      .to_a.should == ["🏳️‍🌈👩‍👩‍👧‍👦"]
   end
 end
